from django import template
from django.conf import settings

from wagtailtrans.models import TranslatablePage

register = template.Library()


@register.simple_tag
def get_canonical_pages_for_delete(page):
    """Get the translations made for this page

    :param page: Page instance
    :return: queryset or False
    """
    page = page.specific
    if (
        settings.WAGTAILTRANS_SYNC_TREE and
        getattr(page, 'language', False) and
        not page.canonical_page
    ):
        return TranslatablePage.objects.filter(canonical_page=page)
    return False



from __future__ import absolute_import, unicode_literals


class TranslationError(Exception):
    """Generic wagtailtrans error."""

    pass


class TranslationMutationError(TranslationError):
    """An error raised while performing a language change.

    This could be for example a page moving from one language to another
    or a tree structure which needs to be synced with another language tree.
    """

    pass



from django.contrib import admin

from wagtailtrans import models


class LanguageAdmin(admin.ModelAdmin):
    list_display = ('code', 'position', 'is_default')


class TranslatablePageAdmin(admin.ModelAdmin):
    list_display = ('title', 'slug', 'language')


admin.site.register(models.Language, LanguageAdmin)
admin.site.register(models.TranslatablePage, TranslatablePageAdmin)



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy
from wagtail.wagtailadmin.views import generic
from wagtail.wagtailcore.permission_policies import ModelPermissionPolicy

from wagtailtrans.forms import LanguageForm
from wagtailtrans.models import Language

language_permission_policy = ModelPermissionPolicy(Language)


class Index(generic.IndexView):
    model = Language
    permission_policy = language_permission_policy
    context_object_name = 'languages'
    template_name = 'wagtailtrans/languages/index.html'
    add_url_name = 'wagtailtrans_languages:add'
    page_title = ugettext_lazy("Languages")
    add_item_label = ugettext_lazy("Add a language")
    header_icon = 'folder-open-1'


class Create(generic.CreateView):
    form_class = LanguageForm
    permission_policy = language_permission_policy
    page_title = ugettext_lazy("Add language")
    success_message = ugettext_lazy("Language '{0}' created.")
    add_url_name = 'wagtailtrans_languages:add'
    edit_url_name = 'wagtailtrans_languages:edit'
    index_url_name = 'wagtailtrans_languages:index'
    header_icon = 'folder-open-1'


class Edit(generic.EditView):
    model = Language
    permission_policy = language_permission_policy
    form_class = LanguageForm
    success_message = ugettext_lazy("Language '{0}' updated.")
    error_message = ugettext_lazy(
        "The language could not be saved due to errors.")
    delete_item_label = ugettext_lazy("Delete language")
    edit_url_name = 'wagtailtrans_languages:edit'
    index_url_name = 'wagtailtrans_languages:index'
    delete_url_name = 'wagtailtrans_languages:delete'
    context_object_name = 'languages'
    header_icon = 'folder-open-1'


class Delete(generic.DeleteView):
    model = Language
    permission_policy = language_permission_policy
    success_message = ugettext_lazy("Language '{0}' deleted.")
    index_url_name = 'wagtailtrans_languages:index'
    delete_url_name = 'wagtailtrans_languages:delete'
    page_title = ugettext_lazy("Delete language")
    confirmation_message = ugettext_lazy(
        "Are you sure you want to delete this language?")
    header_icon = 'folder-open-1'



import logging

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

logger = logging.getLogger()


class WagtailTransConfig(AppConfig):
    label = 'wagtailtrans'
    name = 'wagtailtrans'
    verbose_name = _('Wagtail Translations')

    def ready(self):
        from django.conf import settings

        # check if WAGTAILTRANS_SYNC_TREE is set, if not use default (True)
        if not hasattr(settings, 'WAGTAILTRANS_SYNC_TREE'):
            setattr(settings, 'WAGTAILTRANS_SYNC_TREE', True)
            logger.warning(
                "Setting WAGTAILTRANS_SYNC_TREE undefined. Please specify "
                "WAGTAILTRANS_SYNC_TREE in your projects settings file.")

        from wagtailtrans.signals import register_signal_handlers
        register_signal_handlers()



from __future__ import absolute_import, unicode_literals

from operator import itemgetter

from django import forms
from django.conf import settings
from django.utils.translation import ugettext as _
from wagtail.wagtailcore.models import Page

from wagtailtrans.models import Language, TranslatablePage


class LanguageForm(forms.ModelForm):

    class Meta:
        model = Language
        fields = (
            'code',
            'is_default',
            'position',
            'live',
        )

    def __init__(self, *args, **kwargs):
        super(LanguageForm, self).__init__(*args, **kwargs)

        # Sort language choices according their display name
        sorted_choices = sorted(self.fields['code'].choices, key=itemgetter(1))
        self.fields['code'].choices = sorted_choices

        # Disable `is_default` field when a default language is already set.
        if Language.objects.default() and settings.WAGTAILTRANS_SYNC_TREE:
            self.fields['is_default'].widget.attrs['disabled'] = 'disabled'
            self.fields['is_default'].help_text = _("""
                There can only be one default language, this
                can't be changed when in SYNC mode.""")

    def clean_is_default(self):
        """Force the `is_default` to stay the same, when in sync mode."""
        if (
            self.instance and
            settings.WAGTAILTRANS_SYNC_TREE and
            Language.objects.default()
        ):
            return self.instance.is_default
        return self.cleaned_data['is_default']


class TranslationForm(forms.Form):
    copy_from_canonical = forms.BooleanField(required=False)
    parent_page = forms.ModelChoiceField(
        queryset=TranslatablePage.objects.filter(language__is_default=False))

    def __init__(self, *args, **kwargs):
        self.page = kwargs.pop('page')
        self.site = self.page.get_site()
        self.language = kwargs.pop('language')
        self.base_fields['parent_page'].queryset = self.get_queryset()

        if self._page_has_required(self.page):
            self.base_fields['copy_from_canonical'].initial = True
            self.base_fields['copy_from_canonical'].disabled = True
            self.base_fields['copy_from_canonical'].help_text = _(
                "All fields need to be copied because of some required fields")

        super(TranslationForm, self).__init__(*args, **kwargs)

    def get_queryset(self):
        qs = TranslatablePage.objects.filter(language=self.language)
        allowed_pages = [p.pk for p in qs if (
            self.page.can_move_to(p) and p.get_site() == self.site
        )]
        qs = TranslatablePage.objects.filter(pk__in=allowed_pages)
        if not qs:
            return Page.objects.filter(pk=self.site.root_page.pk)
        return qs

    def _page_has_required(self, page):
        common_fields = set(TranslatablePage._meta.fields)
        specific_fields = set(page.specific._meta.fields) - common_fields

        required_fields = [f for f in specific_fields
                           if not f.blank and not f.name.endswith('ptr')]
        if required_fields:
            return True
        return False



from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from wagtailtrans.views import language

app_name = 'wagtailtrans'

urlpatterns = [
    url(r'^$', language.Index.as_view(), name='index'),
    url(r'^add/$', language.Create.as_view(), name='add'),
    url(r'^(\d+)/$', language.Edit.as_view(), name='edit'),
    url(r'^(\d+)/delete/$', language.Delete.as_view(), name='delete'),
]



from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db.models.signals import post_save, pre_delete
from wagtail.wagtailcore.models import Site, get_page_models

from wagtailtrans.models import Language, TranslatablePage
from wagtailtrans.permissions import (
    create_group_permissions, get_or_create_language_group)


def synchronize_trees(sender, instance, **kwargs):
    """synchronize the translation trees when
    a TranslatablePage is created.

    :param sender: Sender model
    :param instance: TranslatablePage instance
    :param kwargs: kwargs e.g. created

    """
    if (
        not kwargs.get('created') or
        not getattr(instance, 'language', False) or
        not instance.language.is_default
    ):
        return

    try:
        instance.get_site()
    except ObjectDoesNotExist:
        return

    for lang in Language.objects.filter(is_default=False):
        instance.create_translation(language=lang, copy_fields=True)


def synchronize_deletions(sender, instance, **kwargs):
    """We use pre_delete because when sync is disabled the foreign_key on
    canonical pages on_delete is set_null.

    :param sender: Sender model
    :param instance: TranslatablePage Instance
    :param kwargs: kwargs

    """
    language = getattr(instance, 'language', False)
    if language and not instance.canonical_page:
        instance.get_translations(only_live=False).delete()


def create_new_language_tree(sender, instance, **kwargs):
    """Signal will catch creation of a new language
    If sync trees is enabled it will create a whole new tree with
    correlating language.

    :param sender: Sender model
    :param instance: Language instance
    :param kwargs: kwargs e.g. created

    """
    if not kwargs.get('created'):
        return

    for site in Site.objects.all():
        site_pages = site.root_page.get_children().values_list('pk', flat=True)
        canonical_home_page = (
            TranslatablePage.objects
            .filter(pk__in=site_pages, language=Language.objects.default())
            .first())
        if not canonical_home_page:
            # no pages created yet.
            return
        descendants = canonical_home_page.get_descendants(inclusive=True)
        for child_page in descendants:
            child_page.specific.create_translation(instance, copy_fields=True)


def create_language_permissions_and_group(sender, instance, **kwargs):
    """Create a new `Translator` role with it's required permissions.

    :param sender: Sender model
    :param instance: Language instance
    :param kwargs: kwargs e.g. created

    """
    if not kwargs.get('created'):
        return

    group = get_or_create_language_group(instance)
    create_group_permissions(group, instance)


def register_signal_handlers():
    """Registers signal handlers.

    To create a signal for TranslatablePage we have to use wagtails
    get_page_model.

    """
    post_save.connect(create_language_permissions_and_group, sender=Language)

    if settings.WAGTAILTRANS_SYNC_TREE:
        post_save.connect(create_new_language_tree, sender=Language)

        for model in get_page_models():
            if hasattr(model, 'create_translation'):
                post_save.connect(synchronize_trees, sender=model)

            if hasattr(model, 'get_translations'):
                pre_delete.connect(synchronize_deletions, sender=model)



from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from wagtailtrans.views import translation

app_name = 'wagtailtrans'

urlpatterns = [
    url(r'^(?P<page_pk>\w+)/add/(?P<language_code>\w+)/$',
        translation.Add.as_view(), name='add'),
]



from django.conf import settings
from django.contrib.auth.models import Group, Permission
from wagtail.wagtailcore.models import (
    Collection, GroupCollectionPermission, GroupPagePermission,
    PagePermissionTester, UserPagePermissionsProxy)


def create_group_permissions(group, language):
    """create all required permissions on the translator group

    :param group:  Group instance
    :param language: Language instance
    """
    collection_perms = [
        Permission.objects.get_by_natural_key(
            u'add_document', u'wagtaildocs', u'document'),
        Permission.objects.get_by_natural_key(
            u'change_document', u'wagtaildocs', u'document'),
        Permission.objects.get_by_natural_key(
            u'delete_document', u'wagtaildocs', u'document'),
        Permission.objects.get_by_natural_key(
            u'change_image', u'wagtailimages', u'image'),
        Permission.objects.get_by_natural_key(
            u'add_image', u'wagtailimages', u'image'),
        Permission.objects.get_by_natural_key(
            u'delete_image', u'wagtailimages', u'image'),
    ]
    # access wagtail admin permission
    group.permissions.add(Permission.objects.get_by_natural_key(
        u'access_admin', u'wagtailadmin', u'admin'
    ))

    collection = Collection.objects.filter(
        name='collection-%s' % language.code).first()
    if not collection:
        root = Collection.objects.first().get_root()
        collection = root.add_child(name='collection-%s' % language.code)
        for perm in collection_perms:
            GroupCollectionPermission.objects.create(
                permission=perm,
                group=group,
                collection=collection
            )


def get_or_create_language_group(language):
    """Create new translator group for language

    :param language: Language instance
    :return: Group
    """
    group, created = Group.objects.get_or_create(
        name='translator-%s' % language.code)
    if created:
        create_group_permissions(group, language)
    return group


def create_group_page_permission(page, language):
    """Create new GroupPagePermissions for
    the newly created page

    :param page: TranslatablePage instance
    :param language: Language instance
    """
    translator_perms = ['edit', 'publish']
    group = get_or_create_language_group(language)

    for perm in translator_perms:
        GroupPagePermission.objects.create(
            group=group,
            page=page,
            permission_type=perm
        )


class TranslatablePagePermissionTester(PagePermissionTester):
    """Custom permissions tester for translatable pages."""

    def can_delete(self):
        """Check if a page can be deleted
        We make the check if the translated sites are kept in sync and
        if the page is a translated page (it has a canonical page)

        :return: Boolean

        """
        has_canonical = getattr(self.page.specific, 'canonical_page', False)
        if (
            has_canonical and settings.WAGTAILTRANS_SYNC_TREE and
            not self.user.is_superuser
        ):
            return False
        return super(TranslatablePagePermissionTester, self).can_delete()


class TranslatableUserPagePermissionsProxy(UserPagePermissionsProxy):
    """Custom Permission proxy to insert our custom tester"""

    def for_page(self, page):
        """Get the user page permissions for this page
        We implement our custom  Permission tester here

        :param page: Page object
        :return: TranslatablePagePermissionsTester instance

        """
        return TranslatablePagePermissionTester(self, page)



from django.db import models


class LanguageManager(models.Manager):
    """Custom manager for the `Language` model."""

    def live(self):
        """Return all the live languages."""
        return self.filter(live=True)

    def default(self):
        """Return the first choice of default languages."""
        return self.live().filter(is_default=True).first()



from django import forms
from django.forms import HiddenInput
from django.utils.safestring import mark_safe


class ReadOnlyWidget(forms.Select):
    """
    Render the original field as a hidden widget.
    And create a text display for the label
    """
    def __init__(self, text_display, *args, **kwargs):
        self.text_display = text_display
        self.initial_widget = HiddenInput()
        super(ReadOnlyWidget, self).__init__(*args, **kwargs)

    def render(self, *args, **kwargs):
        original_content = self.initial_widget.render(*args, **kwargs)

        return mark_safe("""<span class="hidden">%s</span>%s""" % (
            original_content, self.text_display))



from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.db.models import Q
from django.http import Http404
from django.shortcuts import redirect
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import activate, ugettext_lazy as _
from wagtail.utils.decorators import cached_classmethod
from wagtail.wagtailadmin.edit_handlers import (
    FieldPanel, MultiFieldPanel, ObjectList, PageChooserPanel, TabbedInterface)
from wagtail.wagtailadmin.forms import WagtailAdminPageForm
from wagtail.wagtailcore.models import Page

from .edit_handlers import ReadOnlyWidget
from .managers import LanguageManager
from .permissions import TranslatableUserPagePermissionsProxy


@python_2_unicode_compatible
class Language(models.Model):
    """User defined language."""

    code = models.CharField(
        max_length=12, choices=settings.LANGUAGES, unique=True,
        help_text="One of the languages defined in LANGUAGES")
    is_default = models.BooleanField(
        default=False, help_text="""
        Visitors with no language preference will see the site in
        this language
        """)
    position = models.IntegerField(
        default=0, help_text="""
        Language choices and translations will be displayed in this
        order
        """)
    live = models.BooleanField(
        default=True,
        help_text="Is this language available for visitors to view?")

    objects = LanguageManager()

    class Meta:
        ordering = ['position']

    def __str__(self):
        return self.get_code_display()

    def has_pages_in_site(self, site):
        return (
            self.pages.filter(
                path__startswith=site.root_page.path
            ).exists())


class AdminTranslatablePageForm(WagtailAdminPageForm):
    """Form to be used in the wagtail admin."""

    def __init__(self, *args, **kwargs):
        super(AdminTranslatablePageForm, self).__init__(*args, **kwargs)

        canonical_page_text = _("None")
        if self.instance.canonical_page:
            canonical_page_text = self.instance.canonical_page.title
        self.fields['canonical_page'].widget = ReadOnlyWidget(
            text_display=canonical_page_text)

        language_display = Language.objects.filter(
            pk=self.initial['language']).first()
        if self.instance.is_canonical and language_display:
            language_display = "{} - {}".format(language_display, "canonical")

        self.fields['language'].widget = ReadOnlyWidget(
            text_display=language_display if language_display else '')

    def clean_language(self):
        return (
            self.instance.force_parent_language(self.parent_page) or
            Language.objects.default())


def _language_default():
    return Language.objects.default()


@python_2_unicode_compatible
class TranslatablePage(Page):
    canonical_page = models.ForeignKey(
        'self', related_name='translations', blank=True,
        null=True, on_delete=models.SET_NULL)
    language = models.ForeignKey(
        Language, related_name='pages', on_delete=models.PROTECT,
        default=_language_default)

    translation_panels = [
        MultiFieldPanel([
            FieldPanel('language'),
            PageChooserPanel('canonical_page'),
        ])
    ]

    base_form_class = AdminTranslatablePageForm

    def __str__(self):
        return "{} ({})".format(self.title, self.language)

    def serve(self, request, *args, **kwargs):
        activate(self.language.code)
        return super(TranslatablePage, self).serve(request, *args, **kwargs)

    def move(self, target, pos=None, suppress_sync=False):
        """Move the page to another target.

        :param target: the new target to move the page to
        :param pos: position of the page in the new target
        :param suppress_sync: suppress syncing the translated pages

        """
        super(TranslatablePage, self).move(target, pos)
        if (
            not suppress_sync and
            settings.WAGTAILTRANS_SYNC_TREE and
            self.language.is_default
        ):
            self.move_translated_pages(canonical_target=target, pos=pos)

    def move_translated_pages(self, canonical_target, pos=None):
        """Move only the translated pages of this instance (not self).

        This is only called when WAGTAILTRANS_SYNC_TREE is enabled

        :param canonical_target: Parent of the canonical page
        :param pos: position

        """
        translations = self.get_translations(only_live=False)
        if getattr(canonical_target, 'canonical_page', False):
            canonical_target = canonical_target.canonical_page

        for page in translations:
            # get target because at this point we assume the tree is in sync.
            target = TranslatablePage.objects.filter(
                Q(language=page.language),
                Q(canonical_page=canonical_target) | Q(pk=canonical_target.pk)
            ).get()

            page.move(target=target, pos=pos, suppress_sync=True)

    def get_translations(self, only_live=True, include_self=False):
        """Get translation of this page.

        :param only_live: Boolean to filter on live pages
        :param include_self: Should this page be part of the result set
        :return: TranslatablePage instance

        """
        canonical_page = self.canonical_page or self
        translations = TranslatablePage.objects.filter(
            Q(language__live=True),
            Q(canonical_page=canonical_page) |
            Q(pk=canonical_page.pk)
        )

        if only_live:
            translations = translations.live()

        if not include_self:
            translations = translations.exclude(pk=self.pk)

        return translations.order_by('language__position')

    def has_translation(self, language):
        """Check if page isn't already translated in given language.

        :param language: Language instance
        :return: Boolean

        """
        return language.pages.filter(canonical_page=self).exists()

    def get_translation_parent(self, language):
        site = self.get_site()
        if not language.has_pages_in_site(site):
            return site.root_page

        translation_parent = (
            TranslatablePage.objects
            .filter(
                canonical_page=self.get_parent(),
                language=language,
                path__startswith=site.root_page.path
            ).first())
        return translation_parent

    def create_translation(self, language, copy_fields=False, parent=None):
        """Create a translation for this page. If tree syncing is enabled the
        copy will also be moved to the corresponding language tree.

        :param language: Language instance
        :param copy_fields: Boolean specifying if the content should be copied
        :param parent: Parent page instance for the translation
        :return: new Translated page (or subclass) instance

        """
        if self.has_translation(language):
            raise Exception("Translation already exists")

        if not parent:
            parent = self.get_translation_parent(language)

        slug = '%s-%s' % (self.slug, language.code)
        if self.slug == self.language.code:
            slug = language.code

        update_attrs = {
            'title': self.title,
            'slug': slug,
            'language': language,
            'live': False,
            'canonical_page': self,
        }

        if copy_fields:
            kwargs = {'update_attrs': update_attrs}
            if parent != self.get_parent():
                kwargs['to'] = parent

            new_page = self.copy(**kwargs)
        else:
            model_class = self.content_type.model_class()
            new_page = model_class(**update_attrs)
            parent.add_child(instance=new_page)

        return new_page

    def force_parent_language(self, parent=None):
        """Set Page instance language to the parent language.

        TODO: This used to be called from the `save()` method, but
        afterwards wasn't saved. Saving it would lead to recursion
        errors especially in combination with the defined signal handlers.
        We'll have to see if we can perform `force_parent_language` upon
        save (either by override `save()` or by using
        `pre_save` or `post_save` signals)

        :param parent: Parent page of self
        :return: Language instance

        """
        if not parent:
            parent = self.get_parent()

        parent = parent.specific
        if hasattr(parent, 'language'):
            if self.language != parent.language:
                self.language = parent.language
        return self.language

    @cached_property
    def has_translations(self):
        return TranslatablePage.objects.filter(canonical_page=self).exists()

    @cached_property
    def is_canonical(self):
        return not self.canonical_page and self.has_translations


@cached_classmethod
def get_edit_handler(cls):
    tabs = []
    if cls.content_panels:
        tabs.append(ObjectList(cls.content_panels, heading=_("Content")))
    if cls.promote_panels:
        tabs.append(ObjectList(cls.promote_panels, heading=_("Promote")))
    if cls.translation_panels:
        tabs.append(ObjectList(
            cls.translation_panels, heading=_("Translations")))
    if cls.settings_panels:
        tabs.append(ObjectList(
            cls.settings_panels, heading=_("Settings"), classname='settings'))

    EditHandler = TabbedInterface(tabs, base_form_class=cls.base_form_class)  # noqa
    return EditHandler.bind_to_model(cls)

TranslatablePage.get_edit_handler = get_edit_handler


def get_user_language(request):
    """Get the Language corresponding to a request.
    return default language if Language does not exist in site

    :param request: Request object
    :return: Language instance
    """
    if hasattr(request, 'LANGUAGE_CODE'):
        language = (
            Language.objects
            .live()
            .filter(code=request.LANGUAGE_CODE)
            .first())
        if language:
            return language
    return Language.objects.default()


class TranslatableSiteRootPage(Page):
    """Root page of any translatable site.

    This page should be used as the root page because it will
    route the requests to the right language.

    """

    def serve(self, request, *args, **kwargs):
        """Serve TranslatablePage in the correct language

        :param request: request object
        :return: Http403 or Http404

        """
        language = get_user_language(request)
        candidates = TranslatablePage.objects.live().specific().child_of(self)
        try:
            translation = candidates.filter(language=language).get()
            return redirect(translation.url)
        except TranslatablePage.DoesNotExist:
            raise Http404


def page_permissions_for_user(self, user):
    """Patch for the page permissions adding our custom proxy

    Note: Since wagtail doesn't call this method on the
          specific page we need to patch the default page
          implementation for this.

    :param user: User instance
    :return: user permissions for page

    """
    user_perms = TranslatableUserPagePermissionsProxy(user)
    return user_perms.for_page(self)

Page.permissions_for_user = page_permissions_for_user



from __future__ import absolute_import, unicode_literals

from django.shortcuts import get_object_or_404, redirect
from django.views.generic.edit import FormView
from wagtail.wagtailadmin.edit_handlers import (
    FieldPanel, ObjectList, PageChooserPanel, TabbedInterface)

from wagtailtrans.forms import TranslationForm
from wagtailtrans.models import Language, TranslatablePage


class Add(FormView):
    """View to add a new ``Translation``."""

    form_class = TranslationForm
    template_name = 'wagtailtrans/translation/add.html'

    add_panels = [
        FieldPanel('copy_from_canonical'),
        PageChooserPanel('parent_page'),
    ]

    edit_handler = TabbedInterface([
        ObjectList(add_panels, heading='Translate',
                   base_form_class=TranslationForm),
    ])

    def dispatch(self, request, page_pk, language_code, *args, **kwargs):
        self.page = get_object_or_404(TranslatablePage, pk=page_pk)
        self.language = get_object_or_404(Language, code=language_code)
        return super(Add, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self, *args, **kwargs):
        form_kwargs = super(Add, self).get_form_kwargs(*args, **kwargs)
        form_kwargs.update({
            'page': self.page,
            'language': self.language,
        })
        return form_kwargs

    def form_valid(self, form):
        parent = form.cleaned_data['parent_page']
        copy_from_canonical = form.cleaned_data['copy_from_canonical']

        new_page = self.page.create_translation(
            self.language, copy_fields=copy_from_canonical, parent=parent)

        # new_page.move(parent, pos='last-child')
        return redirect(
            'wagtailadmin_pages:edit', new_page.id)

    def get_context_data(self, *args, **kwargs):
        context = super(Add, self).get_context_data(*args, **kwargs)
        edit_handler = self.edit_handler.bind_to_model(self.page)

        context.update({
            'page': self.page,
            'language': self.language,
            'content_type': self.page.content_type,
            'parent_page': self.page.get_parent(),
            'edit_handler': edit_handler(self.page, context['form']),
        })

        return context



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-30 05:04
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import wagtailtrans.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0029_unicode_slugfield_dj19'),
    ]

    operations = [
        migrations.CreateModel(
            name='Language',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(choices=[('af', 'Afrikaans'), ('ar', 'Arabic'), ('ast', 'Asturian'), ('az', 'Azerbaijani'), ('bg', 'Bulgarian'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('br', 'Breton'), ('bs', 'Bosnian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('cy', 'Welsh'), ('da', 'Danish'), ('de', 'German'), ('dsb', 'Lower Sorbian'), ('el', 'Greek'), ('en', 'English'), ('en-au', 'Australian English'), ('en-gb', 'British English'), ('eo', 'Esperanto'), ('es', 'Spanish'), ('es-ar', 'Argentinian Spanish'), ('es-co', 'Colombian Spanish'), ('es-mx', 'Mexican Spanish'), ('es-ni', 'Nicaraguan Spanish'), ('es-ve', 'Venezuelan Spanish'), ('et', 'Estonian'), ('eu', 'Basque'), ('fa', 'Persian'), ('fi', 'Finnish'), ('fr', 'French'), ('fy', 'Frisian'), ('ga', 'Irish'), ('gd', 'Scottish Gaelic'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hsb', 'Upper Sorbian'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('id', 'Indonesian'), ('io', 'Ido'), ('is', 'Icelandic'), ('it', 'Italian'), ('ja', 'Japanese'), ('ka', 'Georgian'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('kn', 'Kannada'), ('ko', 'Korean'), ('lb', 'Luxembourgish'), ('lt', 'Lithuanian'), ('lv', 'Latvian'), ('mk', 'Macedonian'), ('ml', 'Malayalam'), ('mn', 'Mongolian'), ('mr', 'Marathi'), ('my', 'Burmese'), ('nb', 'Norwegian Bokm\xe5l'), ('ne', 'Nepali'), ('nl', 'Dutch'), ('nn', 'Norwegian Nynorsk'), ('os', 'Ossetic'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Brazilian Portuguese'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sq', 'Albanian'), ('sr', 'Serbian'), ('sr-latn', 'Serbian Latin'), ('sv', 'Swedish'), ('sw', 'Swahili'), ('ta', 'Tamil'), ('te', 'Telugu'), ('th', 'Thai'), ('tr', 'Turkish'), ('tt', 'Tatar'), ('udm', 'Udmurt'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('vi', 'Vietnamese'), ('zh-hans', 'Simplified Chinese'), ('zh-hant', 'Traditional Chinese')], help_text='One of the languages defined in LANGUAGES', max_length=12, unique=True)),
                ('is_default', models.BooleanField(default=False, help_text='\n        Visitors with no language preference will see the site in\n        this language\n        ')),
                ('position', models.IntegerField(default=0, help_text='\n        Language choices and translations will be displayed in this\n        order\n        ')),
                ('live', models.BooleanField(default=True, help_text='Is this language available for visitors to view?')),
            ],
            options={
                'ordering': ['position'],
            },
        ),
        migrations.CreateModel(
            name='TranslatablePage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('canonical_page', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='translations', to='wagtailtrans.TranslatablePage')),
                ('language', models.ForeignKey(default=wagtailtrans.models._language_default, on_delete=django.db.models.deletion.PROTECT, to='wagtailtrans.Language')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='TranslatableSiteRootPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]



from setuptools import find_packages, setup


setup(
    name='wagtailtrans',
    version='0.1.0b3',
    description='A Wagtail add-on for supporting multilingual sites',
    author='Lukkien BV',
    author_email='support@lukkien.com',
    url='https://lukkien.com/',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    license='BSD',
    long_description=open('README.rst').read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
    install_requires=[
        'wagtail>=1.6',
    ]
)



from django.conf import settings
from django.conf.urls import include, url
from django.core import urlresolvers
from wagtail.wagtailadmin import widgets
from wagtail.wagtailadmin.menu import MenuItem
from wagtail.wagtailcore import hooks

from wagtailtrans.models import Language
from wagtailtrans.urls import languages, translations


@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^language/',
            include(languages, namespace='wagtailtrans_languages')),
        url(r'^translate/',
            include(translations, namespace='wagtailtrans_translations')),
    ]


@hooks.register('register_settings_menu_item')
def register_language_menu_item():
    return MenuItem(
        'Languages',
        urlresolvers.reverse('wagtailtrans_languages:index'),
        classnames='icon icon-snippet',
        order=1000,
    )


if not settings.WAGTAILTRANS_SYNC_TREE:
    """Only load hooks when WAGTAILTRANS_SYNC_TREE is disabled"""

    @hooks.register('register_page_listing_buttons')
    def page_translations_menu(page, page_perms, is_parent=False):
        if not hasattr(page, 'language'):
            return

        if hasattr(page, 'canonical_page') and page.canonical_page:
            return

        yield widgets.ButtonWithDropdownFromHook(
            'Translate into',
            hook_name='wagtailtrans_dropdown_hook',
            page=page,
            page_perms=page_perms,
            is_parent=is_parent,
            priority=10)

    @hooks.register('wagtailtrans_dropdown_hook')
    def page_translations_menu_items(page, page_perms, is_parent=False):
        prio = 1
        exclude_lang = None

        if hasattr(page, 'language') and page.language:
            exclude_lang = page.language

        live_languages = Language.objects.filter(live=True)
        if exclude_lang:
            live_languages = live_languages.exclude(pk=exclude_lang.pk)

        page_translations = (
            page.get_translations(only_live=False, include_self=True))

        exclude_pages = page_translations.values_list(
            'language__code', flat=True)
        live_languages = live_languages.exclude(code__in=exclude_pages)

        for language in live_languages:
            lang = [x for x in settings.LANGUAGES if x[0] == language.code][0]

            yield widgets.Button(
                '%s' % lang[1],
                urlresolvers.reverse(
                    'wagtailtrans_translations:add', kwargs={
                        'page_pk': page.pk, 'language_code': language.code}),
                priority=prio)

            prio += 1
